@@ -1,6 +1,6 @@
 --[[
 Copyright (c) 2016, Andrew Lewis <nerf@judo.za.org>
-Copyright (c) 2016, Vsevolod Stakhov <vsevolod@highsecure.ru>
+Copyright (c) 2022, Vsevolod Stakhov <vsevolod@rspamd.com>
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -29,6 +29,7 @@ local rspamd_logger = require "rspamd_logger"
 local ucl = require "ucl"
 local E = {}
 local N = 'metadata_exporter'
+local HOSTNAME = rspamd_util.get_hostname()
 
 local settings = {
   pusher_enabled = {},
@@ -74,11 +75,31 @@ local function get_general_metadata(task, flatten, no_content)
   r.user = task:get_user() or 'unknown'
   r.qid = task:get_queue_id() or 'unknown'
   r.subject = task:get_subject() or 'unknown'
-  r.action = task:get_metric_action('default')
+  r.action = task:get_metric_action()
+  r.rspamd_server = HOSTNAME
 
-  local s = task:get_metric_score('default')[1]
+  local s = task:get_metric_score()[1]
   r.score = flatten and string.format('%.2f', s) or s
 
+  local fuzzy = task:get_mempool():get_variable("fuzzy_hashes", "fstrings")
+  if fuzzy and #fuzzy > 0 then
+    local fz = {}
+    for _,h in ipairs(fuzzy) do
+      table.insert(fz, h)
+    end
+    if not flatten then
+      r.fuzzy = fz
+    else
+      r.fuzzy = table.concat(fz, ', ')
+    end
+  else
+    if not flatten then
+      r.fuzzy = {}
+    else
+      r.fuzzy = ''
+    end
+  end
+
   local rcpt = task:get_recipients('smtp')
   if rcpt then
     local l = {}
@@ -132,6 +153,18 @@ local function get_general_metadata(task, flatten, no_content)
       return 'unknown'
     end
   end
+
+  local scan_real = task:get_scan_time()
+  scan_real = math.floor(scan_real * 1000)
+  if scan_real < 0 then
+    rspamd_logger.messagex(task,
+        'clock skew detected for message: %s ms real sca time (reset to 0)',
+        scan_real)
+    scan_real = 0
+  end
+
+  r.scan_time = scan_real
+
   if not no_content then
     r.header_from = process_header('from')
     r.header_to = process_header('to')
@@ -144,7 +177,7 @@ end
 
 local formatters = {
   default = function(task)
-    return task:get_content()
+    return task:get_content(), {}
   end,
   email_alert = function(task, rule, extra)
     local meta = get_general_metadata(task, true)
@@ -205,27 +238,31 @@ local selectors = {
     return true
   end,
   is_spam = function(task)
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return is_spam(action)
   end,
   is_spam_authed = function(task)
     if not task:get_user() then
       return false
     end
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return is_spam(action)
   end,
   is_reject = function(task)
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return (action == 'reject')
   end,
   is_reject_authed = function(task)
     if not task:get_user() then
       return false
     end
-    local action = task:get_metric_action('default')
+    local action = task:get_metric_action()
     return (action == 'reject')
   end,
+  is_not_soft_reject = function(task)
+    local action = task:get_metric_action()
+    return (action ~= 'soft reject')
+  end,
 }
 
 local function maybe_defer(task, rule)
@@ -306,7 +343,7 @@ local pushers = {
       host = rule.smtp,
       port = rule.smtp_port or settings.smtp_port or 25,
       from = rule.mail_from or settings.mail_from,
-      recipients = extra.mail_targets,
+      recipients = extra.mail_targets or rule.mail_to or settings.mail_to,
       helo = rule.helo or settings.helo,
       timeout = rule.timeout or settings.timeout,
     }, formatted, sendmail_cb)
@@ -447,6 +484,7 @@ if type(settings.rules) ~= 'table' then
       r.defer = settings.defer
       r.selector = settings.pusher_select.redis_pubsub
       r.formatter = settings.pusher_format.redis_pubsub
+      r.timeout = redis_params.timeout
       settings.rules[r.backend:upper()] = r
     end
   end
@@ -462,6 +500,7 @@ if type(settings.rules) ~= 'table' then
       r.defer = settings.defer
       r.selector = settings.pusher_select.http
       r.formatter = settings.pusher_format.http
+      r.timeout = settings.timeout or 0.0
       settings.rules[r.backend:upper()] = r
     end
   end
@@ -479,6 +518,7 @@ if type(settings.rules) ~= 'table' then
       r.smtp_port = settings.smtp_port
       r.email_template = settings.email_template
       r.defer = settings.defer
+      r.timeout = settings.timeout or 0.0
       r.selector = settings.pusher_select.send_mail
       r.formatter = settings.pusher_format.send_mail
       settings.rules[r.backend:upper()] = r
@@ -556,6 +596,7 @@ backend_check.redis_pubsub = function(k, rule)
     settings.rules[k] = nil
   else
     backend_check.default(k, rule)
+    rule.timeout = redis_params.timeout
   end
 end
 setmetatable(backend_check, {
@@ -611,7 +652,7 @@ for k, r in pairs(settings.rules) do
     name = 'EXPORT_METADATA_' .. k,
     type = 'idempotent',
     callback = gen_exporter(r),
-    priority = 10,
-    flags = 'empty',
+    flags = 'empty,explicit_disable,ignore_passthrough',
+    augmentations = {string.format("timeout=%f", r.timeout or 0.0)}
   })
 end
